@@ -1,6 +1,7 @@
 __all__ = [
-    'ResultException', 'ResultType', 'Result', 'Ok', 'Err',
-    'WrappedBase', 'UnwrappedBase', 'wrap_result', 'auto_unwrap', 'unwrap',
+    'ResultException', 'ResultType', 'WrappedBase', 'UnwrappedBase',
+    'Result', 'Ok', 'Err', 'unwrap',
+    'wrap_result', 'auto_unwrap',
 ]
 
 import types
@@ -8,9 +9,9 @@ import asyncio
 import inspect
 import builtins
 import threading
-from typing import Any, Callable, ForwardRef
 from functools import wraps
 from contextlib import contextmanager
+from typing import Any, Callable, ForwardRef
 
 
 # per-thread Result objects of each Result decorated function call
@@ -36,22 +37,22 @@ class ResultType(_ResultType):
             if not isinstance(V, type):
                 raise TypeError(f'Type {V} sould be instance of type')
 
-            T: ResultType = type(f'Ok[{key}]', (Ok,), {
-                '__annotations__': {
-                    'v': V
-                }
-            })
+            T: ResultType = type(
+                f'Ok[{key}]',
+                (Ok,),
+                {'__annotations__': {'v': V}}
+            )
         elif cls is Err or issubclass(cls, Err):
             E = key
 
             if not isinstance(E, type):
                 raise TypeError(f'Type {E} sould be instance of type')
 
-            T: ResultType = type(f'Err[{key}]', (Err,), {
-                '__annotations__': {
-                    'e': E
-                }
-            })
+            T: ResultType = type(
+                f'Err[{key}]',
+                (Err,),
+                {'__annotations__': {'e': E}}
+            )
         elif cls is Result or issubclass(cls, Result):
             if isinstance(key, (tuple, list)):
                 if len(key) == 1:
@@ -73,12 +74,11 @@ class ResultType(_ResultType):
             if not isinstance(E, type):
                 raise TypeError(f'Type {E} sould be instance of type')
 
-            T: ResultType = type(f'Result[{key}]', (Result,), {
-                '__annotations__': {
-                    'v': V,
-                    'e': E,
-                }
-            })
+            T: ResultType = type(
+                f'Result[{key}]',
+                (Result,),
+                {'__annotations__': {'v': V, 'e': E}}
+            )
         else:
             raise TypeError(f'Unsupported type {cls}') # pragma: no cover
 
@@ -95,12 +95,11 @@ class ResultType(_ResultType):
         if not isinstance(E, type):
             raise TypeError(f'Type {E} sould be instance of type')
 
-        T: ResultType = type(f'Result[{V}, {E}]', (Result,), {
-            '__annotations__': {
-                'v': V,
-                'e': E,
-            }
-        })
+        T: ResultType = type(
+            f'Result[{V}, {E}]',
+            (Result,),
+            {'__annotations__': {'v': V, 'e': E}}
+        )
 
         return T
 
@@ -121,8 +120,8 @@ class ResultType(_ResultType):
             (fn := args[0]) and
             (callable(fn) or asyncio.iscoroutinefunction(fn))
         ):
-            V: type = cls.__annotations__['v']
-            E: type = cls.__annotations__['e']
+            V: type = cls.__annotations__.get('v', fn.__annotations__.get('return', Any))
+            E: type = cls.__annotations__.get('e', Exception)
 
             if not isinstance(V, type):
                 raise TypeError(f'Type {V} sould be instance of type')
@@ -529,3 +528,20 @@ def auto_unwrap(obj) -> Any:
         r = _auto_unwrap_func(obj)
     
     return r
+
+
+if __name__ == '__main__':
+    @Result
+    def div(x: float, y: float) -> float:
+        # can raise "ZeroDivisionError: division by zero" exception
+        z: float = x / y
+        return z
+
+
+    match r := div(1.0, 2.0): # Ok[float](0.5)
+        case Ok(v):
+            assert v == 0.5
+        case Err(e):
+            # unreachable
+            # "ZeroDivisionError: division by zero"
+            assert isinstance(e, ZeroDivisionError)
